'''
This software is subject to the license described in the LICENSE_A+SS.txt file 
included with this software distribution. You may not use this file except in compliance 
with this license.

Copyright (c) Dynastream Innovations Inc. 2016
All rights reserved.
'''

import os
import sys
import serial
import time

global module_ant_id
global module_rssi_cal
global serial_port
global jlink_serial

module_rssi_cal = 0xFF                                                                 # Default value (if no calibration was done)
module_ant_id = 0xFFFFFFFF                                                             # Default value
serial_port = ""                                                                       # Holds handle to the opened serial port
jlink_serial = "682078087"                                                             # Set ESN/SNR of the J-Link programmer, check by running nrfjprog.exe -f NRF52 --ids

def error(err_msg, exit_code = -1):
   '''
   Print an error and quit
   '''
   print('ERROR: {}'.format(err_msg))
   sys.exit(exit_code)

def get_ANT_ID():
   '''
   Request the ANT ID value from the module
   '''
   global serial_port
   global module_ant_id

   # Consult the ANT Message Protocol doc for more information on the string below
   request_ant_id_msg = bytearray([0xA4, 0x02, 0x4D, 0x01, 0x61, 0x8B, 0x00, 0x00])

   serial_port.open()
   serial_port.flush()                                                                 # If module was reset, want to ignore the start-up message
 
   serial_port.write(request_ant_id_msg)   
   ant_id_msg = serial_port.read(size=8)

   serial_port.close()

   # Get ANT ID from output
   module_ant_id = ant_id_msg[3:7]
   module_ant_id = module_ant_id[::-1]                                                 # Preserve endianness
   print("Read ANT ID: ", ''.join(["%02X " %ord( x ) for x in module_ant_id]).strip())
   module_ant_id = int(module_ant_id.encode('hex'), 16)

def get_RSSI_Cal():
   '''
   Request the RSSI Calibration Offset value from the module
   '''
   global serial_port
   global module_rssi_cal

   # Consult the ANT Message Protocol doc for more information on the string below
   request_rssi_cal_msg = bytearray([0xA4, 0x03, 0xE1, 0x00, 0xE4, 0x02, 0xA0, 0x00, 0x00])

   serial_port.open()
   serial_port.flush()                                                                 # If module was reset, want to ignore the start-up message
   
   serial_port.write(request_rssi_cal_msg)
   rssi_cal_msg = serial_port.read(size=6)
   
   serial_port.close()

   # Get RSSI Calibration Offset from output
   module_rssi_cal = rssi_cal_msg[4]
   print("Read RSSI Calibration value: ", ''.join(["%02X " %ord( x ) for x in module_rssi_cal]).strip())
   module_rssi_cal = ord(module_rssi_cal)

def program_module():
   '''
   Program module with new software
   '''
   # First make sure values were actually backed up
   if ((module_ant_id == 0xFFFFFFFF) or (module_rssi_cal == 0xFF)):
      error("Values not backed up! Module will not be re-programmed...\n")  

   global jlink_serial

   # Hex files should be in the same location as this script. These hex files are NOT provided with the script.
   app_hex = "application.hex"
   sd_hex = "softdevice.hex"

   if ((not os.path.isfile(app_hex)) or (not os.path.isfile(sd_hex))):
      error("New hex file(s) not found! Module will not be re-programmed...")

   # Delete everything
   os.system("nrfjprog.exe -f NRF52 --snr "+jlink_serial+" --recover")

   # Program  
   os.system("nrfjprog.exe -f NRF52 --snr "+jlink_serial+" --program "+sd_hex+" --verify")
   os.system("nrfjprog.exe -f NRF52 --snr "+jlink_serial+" --program "+app_hex+" --verify")

   reset_module()

def reset_module():
   # Reset, debug reset is necessary, especially if module is locked after
   os.system("nrfjprog.exe -f NRF52 --snr "+jlink_serial+" --debugreset")
   os.system("nrfjprog.exe -f NRF52 --snr "+jlink_serial+" --pinreset")

def write_val_to_memory(val, addr):
   '''
   Writes provided value to location in module memory specified by addr
   '''
   global jlink_serial

   strVal = "0x%x" % val                                                               # make a hex number, e.g. 0x12345678
   os.system("nrfjprog.exe -f NRF52 --snr "+jlink_serial+" --memwr "+addr+" --val "+strVal)


def setup_serial(COMp, baudr):
   '''
   Opens serial interface to communicate with module
   '''
   global serial_port
   serial_port = serial.Serial(COMp, baudr, timeout=1, rtscts=True)
   serial_port.close()  # In case it was left open

def main():
   '''
   Script entry point
   '''
   global module_rssi_cal
   global module_ant_id

   setup_serial("COM16", 57600)                                                        # specify actual COM port here, baud rate is 57600

   # Reset module
   reset_module()
   time.sleep(1)

   # Get existing data
   print("\nBacking up data fields\n")
   get_ANT_ID()
   get_RSSI_Cal()

   # Reprogram module
   print("\nRe-programming module\n")
   program_module()

   # Write data fields back
   print("\nRestoring values\n")
   write_val_to_memory(module_ant_id, "0x10001090")                                    # UICR Customer Register 4
   write_val_to_memory(module_rssi_cal, "0x1000108C")                                  # UICR Customer Register 3

   # Reset module
   reset_module()

   print("\nDone!")

if __name__ == '__main__':
   try:
      main()
   except Exception as err:
      error('Unexpected error: {}'.format(err))
